/****************************************************************************
**
** Copyright (C) 2005-2005 Trolltech AS. All rights reserved.
**
** This file is part of the example classes of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include <QtGui>

#include "MarGridWindow.h"




MarGridWindow::MarGridWindow()
{
  QWidget *w = new QWidget;
  setCentralWidget(w);

  createActions();
  createMenus();  

  

  
  QPushButton *extract  = new QPushButton(tr("Extract"));
  QPushButton *train = new QPushButton(tr("Train"));
  QPushButton *predict = new QPushButton(tr("Predict"));
  playLabel = new QLabel("Hello");
  trainLabel = new QLabel("Train File: \t ./music.mf");
  predictLabel = new QLabel("Predict File: \t ./test.mf");

  QWidget *margrid = new MarGrid();

  connect(this, SIGNAL(trainFile(QString)), margrid, SLOT(setupTrain(QString)));
  connect(this, SIGNAL(predictFile(QString)), margrid, SLOT(setupPredict(QString)));
  connect(this, SIGNAL(playbackMode(bool)), margrid, SLOT(setPlaybackMode(bool)));
  

  QGridLayout *gridLayout = new QGridLayout;
  gridLayout->addWidget(trainLabel, 0,0,1,3);
  gridLayout->addWidget(predictLabel, 1, 0,1,3);
  gridLayout->addWidget(extract, 2, 0);
  gridLayout->addWidget(train, 2, 1);
  gridLayout->addWidget(predict, 2, 2);
  gridLayout->addWidget(playLabel, 3, 0, 1, 3);
  gridLayout->addWidget(margrid, 4, 0, 1, 3);

  connect(extract, SIGNAL(clicked()), margrid, SLOT(extract()));
  connect(train, SIGNAL(clicked()), margrid, SLOT(train()));
  connect(predict, SIGNAL(clicked()), margrid, SLOT(predict()));
  connect(margrid, SIGNAL(playingFile(QString)), this, SLOT(playingFile(QString))); 


  w->setLayout(gridLayout);

}


void
MarGridWindow::playingFile(QString s) 
{
  playLabel->setText(s);
}

void
MarGridWindow::createMenus()
{
  fileMenu = menuBar()->addMenu(tr("&File"));
  fileMenu->addAction(openTrainAct);
  fileMenu->addAction(openPredictAct);
  fileMenu->addAction(playbackAct);
  
  menuBar()->addSeparator();
  helpMenu = menuBar()->addMenu(tr("&Help"));
  helpMenu->addAction(aboutAct);

  
}


void 
MarGridWindow::openTrainFile()
{
  QString fileName = QFileDialog::getOpenFileName(this);
  cout << "fileName = " << fileName.toStdString() << endl;
  trainLabel->setText("TrainFile: \t" + fileName);
  emit trainFile(fileName);
}


void 
MarGridWindow::openPredictFile()
{
  QString fileName = QFileDialog::getOpenFileName(this);
  predictLabel->setText("PredictFile: \t" + fileName);
  emit predictFile(fileName);
}




void 
MarGridWindow::createActions()
{
  
  openTrainAct = new QAction(tr("&Open Collection File for Training"), this);
  openTrainAct->setStatusTip(tr("Open Collection File for Training"));
  connect(openTrainAct, SIGNAL(triggered()), this, SLOT(openTrainFile()));
  
  openPredictAct = new QAction(tr("&Open Collection File for Predicting"), this);
  openPredictAct->setStatusTip(tr("Open Collection File for Prediciting"));
  connect(openPredictAct, SIGNAL(triggered()), this, SLOT(openPredictFile()));

  // connect(openAct, SIGNAL(triggered()), this, SLOT(open()));
  aboutAct = new QAction(tr("&About"), this);
  aboutAct->setStatusTip(tr("Show the application's About box"));
  connect(aboutAct, SIGNAL(triggered(bool)), this, SLOT(about()));



  playbackAct = new QAction(tr("&Continuous Playback mode"), this);
  playbackAct->setStatusTip(tr("Continuous Playback mode"));
  playbackAct->setCheckable(true);
  connect(playbackAct, SIGNAL(toggled(bool)), this, SIGNAL(playbackMode(bool)));
}


	


void 
MarGridWindow::about()
{
  QMessageBox::about(this, tr("Marsyas MarGrid"),  tr("Marsyas MarGrid: Demonstrates continuous-feedback \n content-based music browsing using Self Organizing Maps"));
  
}
