/*
** Copyright (C) 1998-2005 George Tzanetakis <gtzan@cs.uvic.ca>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#ifndef MARSYAS_EV_GETUPD_H
#define MARSYAS_EV_GETUPD_H

#include <string>
#include <iostream>

#include "MarControl.h"
#include "MarEvent.h"

namespace Marsyas
{
/**
   \class MarEvent
   \ingroup Scheduler
   \brief MarEvent
*/


class MarSystem; // forward declaration

class EvGetUpd : public MarEvent {
protected:
    MarSystem* source_;
    MarSystem* target_;
    std::string src_cname_;
    std::string tgt_cname_;

public:
    // Constructors
    EvGetUpd(MarSystem* src, std::string scname, MarSystem* tgt, std::string tcname);
    EvGetUpd(EvGetUpd& e);
    virtual ~EvGetUpd();

    // Set/Get methods
    std::string getSrcCName() const;
    std::string getTgtCName() const;
    MarSystem* getSource() const;
    MarSystem* getTarget() const;

    void setSrcCName(std::string cname);
    void setTgtCName(std::string cname);
    void setSource(MarSystem* ms);
    void setTarget(MarSystem* ms);
    void setEvent(MarSystem* src, std::string scname, MarSystem* tgt, std::string tcname);

    // Event dispatch
    void dispatch();

    virtual EvGetUpd* clone();

    // the usual stream IO 
    friend std::ostream& operator<<(std::ostream&, MarEvent&);
    friend std::istream& operator>>(std::istream&, MarEvent&);
};

}//namespace Marsyas

#endif
