/*
** Copyright (C) 1998-2005 George Tzanetakis <gtzan@cs.uvic.ca>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include "TmSampleCount.h"
#include "MarSystem.h"
#include "Scheduler.h"

using namespace std;
using namespace Marsyas;

TmSampleCount::TmSampleCount() : TmTimer("TmSampleCount","Virtual")
{
    setReadCtrl(NULL,"mrs_natural/inSamples");
}
TmSampleCount::TmSampleCount(string name) : TmTimer("TmSampleCount",name)
{
    setReadCtrl(NULL,"mrs_natural/inSamples");
}
TmSampleCount::TmSampleCount(MarSystem* ms, string cname) : TmTimer("TmSampleCount","Virtual")
{
    setReadCtrl(ms,cname);
}
TmSampleCount::TmSampleCount(Scheduler* s, MarSystem* ms, string cname) : TmTimer("TmSampleCount","Virtual")
{
    setScheduler(s);
    setReadCtrl(ms,cname);
}
TmSampleCount::TmSampleCount(const TmSampleCount& s) : TmTimer(s)
{
//    setScheduler(s.scheduler);
    setReadCtrl(s.read_src_,s.read_cname_);
}
TmSampleCount::~TmSampleCount(){ }

TmTimer* TmSampleCount::clone() 
{ 
  return new TmSampleCount(*this); 
}

void TmSampleCount::setReadCtrl(MarSystem* ms, string cname)
{
    read_src_=ms;
    read_cname_=cname;
    if (read_src_!=NULL) read_ctrl_=read_src_->getctrl(cname);
}
void TmSampleCount::setSource(MarSystem* ms) { setReadCtrl(ms,read_cname_); }
void TmSampleCount::setSourceCtrl(string cname) { setReadCtrl(read_src_,cname); }

mrs_natural TmSampleCount::readTimeSrc()
{
    if (read_src_==NULL) {
        MRSWARN("TmSampleCount::readTimeSrc()  time source is NULL");
        return 0;
    }
//    mrs_natural m = (read_src_->getctrl(read_cname_)).toNatural();
    mrs_natural m = read_ctrl_->to<mrs_natural>();
    return m;
//    return (read_src_->getctrl(read_cname_)).toNatural() + getTime();
}
void TmSampleCount::trigger()
{
    scheduler->dispatch();
}
mrs_natural TmSampleCount::intervalsize(string interval)
{
    return (read_src_==NULL) ? 0 :
        time2samples(interval,read_src_->getctrl("mrs_real/israte")->to<mrs_real>());
}
void
TmSampleCount::updtimer(std::string cname, TmControlValue value)
{
    bool type_error=false;
    if (cname=="MarSystem/source") {
        if (value.getType()==tmcv_marsystem) { setSource(value.toMarSystem()); }
        else type_error=true;
    }
    else if (cname=="mrs_string/control") {
        if (value.getType()==tmcv_string) { setSourceCtrl(value.toString()); }
        else type_error=true;
    }
    else
        MRSWARN("TmSampleCount::updtimer(string,TmControlValue)  unsupported control");
    if (type_error)
        MRSWARN("TmSampleCount::updtimer(string,TmControlValue)  wrong type to "+cname);
}

